!*******************************************************************************
!  Copyright(C) 2003-2011 Intel Corporation. All Rights Reserved.
!  
!  The source code, information  and  material ("Material") contained herein is
!  owned  by Intel Corporation or its suppliers or licensors, and title to such
!  Material remains  with Intel Corporation  or its suppliers or licensors. The
!  Material  contains proprietary information  of  Intel or  its  suppliers and
!  licensors. The  Material is protected by worldwide copyright laws and treaty
!  provisions. No  part  of  the  Material  may  be  used,  copied, reproduced,
!  modified, published, uploaded, posted, transmitted, distributed or disclosed
!  in any way  without Intel's  prior  express written  permission. No  license
!  under  any patent, copyright  or  other intellectual property rights  in the
!  Material  is  granted  to  or  conferred  upon  you,  either  expressly,  by
!  implication, inducement,  estoppel or  otherwise.  Any  license  under  such
!  intellectual  property  rights must  be express  and  approved  by  Intel in
!  writing.
!  
!  *Third Party trademarks are the property of their respective owners.
!  
!  Unless otherwise  agreed  by Intel  in writing, you may not remove  or alter
!  this  notice or  any other notice embedded  in Materials by Intel or Intel's
!  suppliers or licensors in any way.
!
!*******************************************************************************
!  Content:
!    stream2memory functions  Example Program Text
!*******************************************************************************

      include 'mkl_vsl.f90'
      include "errcheck.inc"

      program MKL_VSL_TEST

      use MKL_VSL_TYPE
      USE MKL_VSL

      type (VSL_STREAM_STATE) :: stream_orig
      type (VSL_STREAM_STATE) :: stream_load
      integer DN, N
      parameter (DN=101)
      parameter ( N= 10)

      integer brng,method,seed

      real(kind=8) ad, bd
      real(kind=8) rd_orig(DN),rd_load(DN)

      integer membufsz
      parameter (membufsz=10000);
      integer(kind=1) membuf(membufsz);

      integer(kind=4) i, nerrors
      integer(kind=4) errcode

      ad = 0.0
      bd = 1.0
      seed=7777777
      brng=VSL_BRNG_SFMT19937
      method = VSL_RNG_METHOD_GAUSSIAN_BOXMULLER2

!     **** Create the original stream to be saved in a memory ****
      errcode = vslnewstream( stream_orig, brng, seed )
      call CheckVslError(errcode)

!     **** Generate random numbers using original stream ****
      errcode = vdrnggaussian(method, stream_orig, DN, rd_orig, ad, bd)
      call CheckVslError( errcode )

!     **** Save stream to memory ****
!     **** Call function getstreamsize to compute memory size ****
!     **** to hold the stream ****
      errcode = vslsavestreamm( stream_orig, membuf )
      call CheckVslError( errcode )

!     **** Generate random numbers using original stream ****
      errcode = vdrnggaussian(method, stream_orig, DN, rd_orig, ad, bd)
      call CheckVslError( errcode )

!     **** Load stream from the memeory ****
      errcode = vslloadstreamm( stream_load, membuf )
      call CheckVslError( errcode )

!     **** Generate random numbers using loaded stream *****
      errcode = vdrnggaussian(method, stream_load, DN, rd_load, ad, bd)
      call CheckVslError( errcode )

!     **** Compare random numbers from original and loaded stream. ****
!     **** Must be identical                                       ****
       print *,"Gaussian numbers:"
      do i=1,N
          print '(A,I2,A,F8.5,A,I2,A,F8.5)'," rd_orig[",i,"]=",         &
     &          rd_orig(i),"     rd_load[",i,"]=",rd_load(i)
      end do

      nerrors = 0
      do i=1,DN
          if ( rd_orig(i) /= rd_load(i) ) then
!             **** Here if results are not identical ****
              nerrors = nerrors + 1
          end if
      end do

      if (nerrors > 0) then
!         **** Here if results are not identical ****
          print *, "ERROR: Loaded stream differs from original stream."
          stop 1
      end if

!     **** Delete original stream ****
      errcode = vsldeletestream(stream_orig)
      call CheckVslError( errcode )

!     **** Delete stream loaded from memory ****
      errcode = vsldeletestream(stream_load)
      call CheckVslError( errcode )

      print *, "PASS: Loaded stream identical with original stream."

      end program
