program test
use matmod
use tmpmatmod
use sj_timer
implicit none
include '../include/sizes'
include '../include/common'
!
!------------------------------------------------------------------------------------
!   File Channels
! 
!  Unit        Filename           Format      Purpose
!   5          user-defined        yes        input file 
!   6          user-defined        yes        general output information
!   7          occupation.dat      yes        occupation number
!   8          population.dat      yes        occupation number by sde
!   9          rhoavg.data         no         rhoavg(nrho,nrho,ntime)
!  10          leadcorr.data       yes        lead correlation functions
!------------------------------------------------------------------------------------
!
character*10            :: date, time
integer                 :: istat, ni, nj, nk 
integer                 :: nballs, nways
integer                 :: ntmp1
logical                 :: lread1, lread2
real*8                  :: cpu1, cpu2
real*8                  :: dtmp1, dtmp2, dtmp3, dtmp4
real*8                  :: occu, occd, occ, occ1, occ2
complex*16              :: ctmp1
!
namelist / para / iorb1
namelist / rho01 / lread1
namelist / rho02 / lread2
!
call cpu_time(cpu1)
call sj_timer_start(1)
!
call genoutput
!
call get_omp_env
!
! set some global variables
!
iorb1 = 9
rewind(5)
read(5, para, end=100)
100 continue
!
call evaluatepara
call allocatememory
call allocate_tmpmem_omp
!
write(6,*)
write(6,*)' number of signs                 : ', nsgn
write(6,*)' number of energy levels         : ', norbs
write(6,*)' number of spin considered       : ', nspin
write(6,*)'     index of orb-1              : ', iorb1
write(6,*)' dimension of orb-1 subsystem    : ', nsys
write(6,*)' dimension of total system       : ', nrho
write(6,*)' dimension of ladder             : ', nstate
write(6,*)
!write(6,*)' ndraw = ', ndraw, ' maxways = ', maxways
write(6,*)' nopr = ', nopr
call flush(6)
!
if (lopen) call setlead1 
if (lopen) call setlead2
!
rho1 = czero
if (nspin .eq. 1) then
   rho1(1,1) = dcmplx(0.0d0, 0.d0)
   rho1(2,2) = dcmplx(1.0d0, 0.d0)
else 
   rho1(4,4) = dcmplx(1.d0, 0.d0)
end if
lread1 = .true.
rewind(5)
read(5, rho01, end=101)
101 read(5,*) ((dmtmp1(ni,nj), ni=1,nsys), nj=1,nsys)
rho1(1:nsys,1:nsys) = dcmplx(dmtmp1(1:nsys,1:nsys), 0.d0)
write(6,*)
write(6,*)'main: matrix rho1 '
call cmatout(nsys, nsys, rho1, dmtmp1(1,1))
write(6,*)
!
if (.not. lopen) then
   rho2 = czero
   if (nspin .eq. 1) then
      rho2(1,1) = dcmplx(1.0d0, 0.d0)
      rho2(2,2) = dcmplx(0.0d0, 0.d0)
   else
      rho2(1,1) = dcmplx(1.d0, 0.d0)
   end if
   lread2 = .true.
   rewind(5)
   read(5, rho02, end=102)
   102 read(5,*) ((dmtmp1(ni,nj), ni=1,nsys), nj=1,nsys)
   rho2(1:nsys,1:nsys) = dcmplx(dmtmp1(1:nsys,1:nsys), 0.d0)
   write(6,*)
   write(6,*)'main: matrix rho2 '
   call cmatout(nsys, nsys, rho2, dmtmp1(1,1))
end if
!
write(6,*)
write(6,*)'main: solving SDE '
call flush(6)

call volt
if (lopen) then
   call sde_euler17_open
else 
   call sde_euler18
end if

call flush(6)
call free_tmpmem_omp
call freememory
!
call cpu_time(cpu2)
!
write(6,*)
write(6,*)' calculation done!             '
write(6,*)' cpu time (overall)            ', cpu2 - cpu1
!
call sj_timer_stop(1)
call date_and_time(date, time)
write(6,*)
write(6,1002)date(7:8),date(5:6),date(1:4),time(1:2),time(3:4),time(5:6)
1002 format(/,' Job finished on ', /,                                   &
            ' ',A2,"/",A2,"/",A4,"  ",A2,":",A2,":",A2,/)
!
call flush(6)
close(5)
518 format(f12.4, 2x, 32(e15.6e3, 1x))
!
end program test
