subroutine zxrhog(sidea, updn, korbs, zalpha, b, zbeta, c)
!
! Creation and annihilation operators are in sparse matrix form
! only nams nonzero matrix elements, being either +1 or -1.
! This subroutine calculates creation/annihilation matrix multiplied
! by genenral complex matrix b, resulting in complex matrix c.
! The matrix multiplication only involves nonzero elements of a and a^dag.
! In doing so the computation cost reduces from N^3 to nams * N
!
use matmod
use tmpmatmod
use sdemod2
use sdemod3
implicit none
include '../include/sizes'
include '../include/common'
!
character*1, intent(in)   :: sidea, updn
integer,    intent(in)    :: korbs
complex*16, intent(in)    :: zalpha, zbeta, b(nrhog,nrhog,*)
complex*16, intent(inout) :: c(nrhog,nrhog,*)
!
integer  :: ni, nj, nk
logical  :: lsame
external :: lsame
logical  :: lefta, righta, lupa, ldowna
complex*16 :: ctmp1
!
if (korbs .le. 0 .or. korbs .gt. nopr) then
   write(6,*)'zxrhog: error! unknown korbs ', korbs
   stop
end if
!
lefta  = lsame(sidea,  'L')
righta = lsame(sidea,  'R')
lupa   = lsame(updn,   'U')
ldowna = lsame(updn,   'D')
! 
if ( (.not. lefta) .and. (.not. righta) ) then
  write(6,*)
  write(6,*)' error input for sidea in zxrhog', sidea
  stop
end if
if ( (.not. lupa) .and. (.not. ldowna) ) then
  write(6,*)
  write(6,*)' error input for updn in zxrhog', updn
  stop
end if
!
c(1:nrhog,1:nrhog,1:mways) = zbeta * c(1:nrhog,1:nrhog,1:mways)
!
if (lupa) then
   if (lefta) then
      do ni=1,mways  
         ctmp1 = zalpha * dfront_up(korbs,ni)
         nj = inup(korbs,ni)
         if (nj .lt. 0) cycle
         c(1:nrhog,1:nrhog,nj) = c(1:nrhog,1:nrhog,nj) + ctmp1 * b(1:nrhog,1:nrhog,ni)
         end do
   else 
      do ni=1,mways
         ctmp1 = zalpha * drear_up(korbs,ni)
         nj = inup(korbs,ni)
         if (nj .lt. 0) cycle
         c(1:nrhog,1:nrhog,nj) = c(1:nrhog,1:nrhog,nj) + ctmp1 * b(1:nrhog,1:nrhog,ni)
         end do
   end if
else
   if (lefta) then
      do ni=1,mways  
         ctmp1 = zalpha * dfront_dn(korbs,ni)
         nj = indn(korbs,ni)
         if (nj .lt. 0) cycle
         c(1:nrhog,1:nrhog,nj) = c(1:nrhog,1:nrhog,nj) + ctmp1 * b(1:nrhog,1:nrhog,ni)
         end do
   else 
      do ni=1,mways
         ctmp1 = zalpha * drear_dn(korbs,ni)
         nj = indn(korbs,ni)
         if (nj .lt. 0) cycle
         c(1:nrhog,1:nrhog,nj) = c(1:nrhog,1:nrhog,nj) + ctmp1 * b(1:nrhog,1:nrhog,ni)
    end do
   end if
end if
!
return
end subroutine zxrhog
